﻿# (c) by Chrisber - this is part of Mapload
import es, parser
from os import sep
reload(parser)



#
# Information
#
info = es.AddonInfo()
info.author = "Chrisber"
info.name = "Mapload"
info.basename = "mapload"
info.version = "3.0"
info.url = "http://addons.eventscripts.com/addons/view/mapload"
info.description = "Allows you a flexible configuration for per-map-scripts and per-map-configs" 



#
# Globals
#
rules = []
matched = []



#
# Events
#
def load():
	es.ServerVar("mapload", info.version).makepublic()
	
	data = parser.Configuration()
	
	try:
		data.parse(es.getString("eventscripts_gamedir") + sep + "cfg" + sep + "mapload.cfg")
	except:
		es.unload("mapload")
		return
	
	for rule in data.data.itervalues():
		rules.append(Rule(rule))
	
	callRules(es.getString("eventscripts_currentmap"))

def unload():
	es.set("mapload", info.version + " (unloaded)")

def es_map_start(event_var):
	callRules(event_var["mapname"])



#
# Functions
#
def callRules(mapname):
	mapname = mapname.strip()
	
	for rule in rules:
		if rule.doesMatch(mapname):
			matched.append(rule)
	
	for rule in rules:
		rule.unloadCheck(mapname)
	
	for rule in rules:
		rule.loadCheck(mapname)
	
	matched.clear()



#
# Classes
#
class Rule:
	ORDERS = ((self.loadScripts, self.loadConfigs), (self.loadConfigs, self.loadScripts))
	
	def __init__(self, rule):
		def get(key, default):
			if key in rule:
				return rule[key]
			
			return default
		
		def getInt(key, default, min, max):
			result = get(key, default)
			
			try:
				result = int(result)
			except:
				result = default
			
			if result < min:
				result = min
			elif result > max:
				result = max
			
			return result
		
		self.prefixes = get("prefixes", "").split(",")
		self.maps = get("maps", "").split(",")
		self.scripts = get("scripts", "").split(",")
		self.configs = get("configs", "").split(",")
		self.order = getInt("order", 0, 0, 1)
		self.unload = getInt("unload", 1, 0, 3)
		self.loaded = False
		self.match = False
	
	def doesMatch(self, mapname):
		self.match = (len(filter(lambda prefix: mapname.startswith(prefix), self.prefixes)) or mapname in self.maps)
		return self.match
	
	def loadCheck(self):
		if not self.loaded and self.match:
			for loading in self.ORDERS[self.order]:
				loading()
	
	def unloadCheck(self):
		if self.loaded and self.unload != 0:
			if self.unload == 1:
				self.unloadScripts()
			elif self.unload == 2:
				if not self.match:
					self.unloadScripts()
			elif len(matched) > 1 or self not in matched:
				self.unloadScripts()
	
	def loadScripts(self, function):
		for script in self.scripts:
			es.load(script)
	
	def unloadScripts(self):
		for script in self.scripts:
			es.unload(script)
	
	def loadConfigs(self):
		for config in self.configs:
			es.server.queuecmd("exec " + config)